@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -32,22 +32,16 @@ For complete examples of this technique, see
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from ...util.callback_manager import _check_callback
-from .handler import Handler
+from .handler import Handler, handle_exception
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -86,7 +80,7 @@ class FunctionHandler(Handler):
 
     '''
 
-    def __init__(self, func):
+    def __init__(self, func, *, trap_exceptions=False):
         '''
 
         Args:
@@ -102,12 +96,16 @@ class FunctionHandler(Handler):
                 and it  should return the passed-in document after making any
                 modifications in-place.
 
+            trap_exceptions (bool) : should exceptions in `func` be caught and
+                logged or allowed to propagate
+
         '''
-        super(FunctionHandler, self).__init__()
+        super().__init__()
 
         _check_callback(func, ('doc',))
 
         self._func = func
+        self._trap_exceptions = trap_exceptions
         self._safe_to_fork = True
 
     # Properties --------------------------------------------------------------
@@ -130,8 +128,15 @@ class FunctionHandler(Handler):
         ``False``.
 
         '''
-        self._func(doc)
-        self._safe_to_fork = False
+        try:
+            self._func(doc)
+        except Exception as e:
+            if self._trap_exceptions:
+                handle_exception(self, e)
+            else:
+                raise
+        finally:
+            self._safe_to_fork = False
 
 #-----------------------------------------------------------------------------
 # Private API
