@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,9 +12,7 @@ to a Bokeh server.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -22,11 +20,7 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-
-# External imports
-from tornado import gen
-
-# Bokeh imports
+from enum import Enum, auto
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -36,6 +30,7 @@ __all__ = (
     'CONNECTED_BEFORE_ACK',
     'CONNECTED_AFTER_ACK',
     'DISCONNECTED',
+    'ErrorReason',
     'NOT_YET_CONNECTED',
     'WAITING_FOR_REPLY',
 )
@@ -48,46 +43,78 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-class NOT_YET_CONNECTED(object):
+class ErrorReason(Enum):
+    NO_ERROR        = auto()
+    HTTP_ERROR      = auto()
+    NETWORK_ERROR   = auto()
+
+class NOT_YET_CONNECTED:
     ''' The ``ClientConnection`` is not yet connected.
 
     '''
 
-    @gen.coroutine
-    def run(self, connection):
-        yield connection._connect_async()
+    async def run(self, connection):
+        return await connection._connect_async()
 
-class CONNECTED_BEFORE_ACK(object):
+class CONNECTED_BEFORE_ACK:
     ''' The ``ClientConnection`` connected to a Bokeh server, but has not yet
     received an ACK from it.
 
     '''
 
-    @gen.coroutine
-    def run(self, connection):
-        yield connection._wait_for_ack()
+    async def run(self, connection):
+        return await connection._wait_for_ack()
 
-class CONNECTED_AFTER_ACK(object):
+class CONNECTED_AFTER_ACK:
     ''' The ``ClientConnection`` connected to a Bokeh server, and has
     received an ACK from it.
 
     '''
 
-    @gen.coroutine
-    def run(self, connection):
-        yield connection._handle_messages()
+    async def run(self, connection):
+        return await connection._handle_messages()
 
-class DISCONNECTED(object):
+class DISCONNECTED:
     ''' The ``ClientConnection`` was connected to a Bokeh server, but is
     now disconnected.
 
     '''
 
-    @gen.coroutine
-    def run(self, connection):
-        raise gen.Return(None)
+    def __init__(self, reason=ErrorReason.NO_ERROR, error_code=None, error_detail=""):
+        ''' Constructs a DISCONNECT-State with given reason (``ErrorReason``
+        enum), error id and additional information provided as string.
 
-class WAITING_FOR_REPLY(object):
+        '''
+        self._error_code = error_code
+        self._error_detail = error_detail
+        self._error_reason = reason
+
+
+    @property
+    def error_reason(self):
+        ''' The reason for the error encoded as an enumeration value.
+
+        '''
+        return self._error_reason
+
+    @property
+    def error_code(self):
+        ''' Holds the error code, if any. None otherwise.
+
+        '''
+        return self._error_code
+
+    @property
+    def error_detail(self):
+        ''' Holds the error message, if any. Empty string otherwise.
+
+        '''
+        return self._error_detail
+
+    async def run(self, connection):
+        return None
+
+class WAITING_FOR_REPLY:
     ''' The ``ClientConnection`` has sent a message to the Bokeh Server which
     should generate a paired reply, and is waiting for the reply.
 
@@ -106,16 +133,15 @@ class WAITING_FOR_REPLY(object):
         ''' The request ID of the originating message. '''
         return self._reqid
 
-    @gen.coroutine
-    def run(self, connection):
-        message = yield connection._pop_message()
+    async def run(self, connection):
+        message = await connection._pop_message()
         if message is None:
-            yield connection._transition_to_disconnected()
+            return await connection._transition_to_disconnected(DISCONNECTED(ErrorReason.NETWORK_ERROR))
         elif 'reqid' in message.header and message.header['reqid'] == self.reqid:
             self._reply = message
-            yield connection._transition(CONNECTED_AFTER_ACK())
+            return await connection._transition(CONNECTED_AFTER_ACK())
         else:
-            yield connection._next()
+            return await connection._next()
 
 #-----------------------------------------------------------------------------
 # Private API
