@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,9 +12,7 @@ line application.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -23,11 +21,8 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 from abc import ABCMeta, abstractmethod
-
-# External imports
-
-# Bokeh imports
-from bokeh.util.future import with_metaclass
+from argparse import ArgumentParser, Namespace
+from typing import Union
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -45,7 +40,7 @@ __all__ = (
 # Dev API
 #-----------------------------------------------------------------------------
 
-class Subcommand(with_metaclass(ABCMeta)):
+class Subcommand(metaclass=ABCMeta):
     ''' Abstract base class for subcommands
 
     Subclasses should implement an ``invoke(self, args)`` method that accepts
@@ -95,7 +90,12 @@ class Subcommand(with_metaclass(ABCMeta)):
 
     '''
 
-    def __init__(self, parser):
+    # specifying static typing of instance attributes
+    # see https://stackoverflow.com/a/51191130
+    name: str
+    help: str
+
+    def __init__(self, parser: ArgumentParser) -> None:
         ''' Initialize the subcommand with its parser
 
         Args:
@@ -116,13 +116,18 @@ class Subcommand(with_metaclass(ABCMeta)):
             self.parser.add_argument(*flags, **arg[1])
 
     @abstractmethod
-    def invoke(self, args):
+    def invoke(self, args: Namespace) -> Union[bool, None]:
         ''' Takes over main program flow to perform the subcommand.
 
         *This method must be implemented by subclasses.*
+        subclassed overwritten methods return different types:
+        bool: Build
+        None: FileOutput (subclassed by HTML, SVG and JSON. PNG overwrites FileOutput.invoke method), Info, Init, \
+                Sampledata, Secret, Serve, Static
+
 
         Args:
-            args (seq) : command line arguments for the subcommand to parse
+            args (argparse.Namespace) : command line arguments for the subcommand to parse
 
         Raises:
             NotImplementedError
