@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,10 +19,10 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-
-# External imports
+from typing import List, Type
 
 # Bokeh imports
+from ..subcommand import Subcommand
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -46,12 +44,11 @@ __all__ = (
 # Private API
 #-----------------------------------------------------------------------------
 
-def _collect():
+def _collect() -> List[Type[Subcommand]]:
     from importlib import import_module
     from os import listdir
     from os.path import dirname
-    from ..subcommand import Subcommand
-
+    # reference type by module as fully
     results = []
 
     for file in listdir(dirname(__file__)):
@@ -65,7 +62,7 @@ def _collect():
         for name in dir(mod):
             attr = getattr(mod, name)
             if isinstance(attr, type) and issubclass(attr, Subcommand):
-                if not hasattr(attr, 'name'): continue # excludes abstract bases
+                if not getattr(attr, 'name', None): continue  # instance attribute not defined on abstract base class
                 results.append(attr)
 
     results = sorted(results, key=lambda attr: attr.name)
