@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,19 +11,19 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+from argparse import Namespace
 
-# External imports
+# Bokeh imports
+from bokeh.ext import init
 
 # Bokeh imports
 from ..subcommand import Subcommand
-from bokeh.ext import init
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -69,9 +69,9 @@ class Init(Subcommand):
         )),
     )
 
-    def invoke(self, args):
+    def invoke(self, args: Namespace) -> bool:
         return init(args.base_dir, interactive=args.interactive,
-            bokehjs_version=args.bokehjs_version, debug=args.debug)
+                    bokehjs_version=args.bokehjs_version, debug=args.debug)
 
 #-----------------------------------------------------------------------------
 # Dev API
