@@ -1,10 +1,10 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-''' Provide base classes for the Bokeh property system.
+""" Provide base classes for the Bokeh property system.
 
 .. note::
     These classes form part of the very low-level machinery that implements
@@ -12,14 +12,12 @@
     classes or their methods will be applicable to any standard usage or to
     anyone who is not directly developing on Bokeh's own infrastructure.
 
-'''
+"""
 
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -27,11 +25,11 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-from copy import copy
 import types
+from copy import copy
+from typing import Any, Type, Union
 
 # External imports
-from six import string_types
 import numpy as np
 
 # Bokeh imports
@@ -40,6 +38,7 @@ from ...util.string import nice_join
 from ..has_props import HasProps
 from .descriptor_factory import PropertyDescriptorFactory
 from .descriptors import BasicPropertyDescriptor
+from .singletons import Intrinsic, Undefined
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -67,13 +66,12 @@ class DeserializationError(Exception):
     pass
 
 class Property(PropertyDescriptorFactory):
-    ''' Base class for Bokeh property instances, which can be added to Bokeh
+    """ Base class for Bokeh property instances, which can be added to Bokeh
     Models.
 
     Args:
-        default (obj or None, optional) :
-            A default value for attributes created from this property to
-            have (default: None)
+        default (obj, optional) :
+            A default value for attributes created from this property to have.
 
         help (str or None, optional) :
             A documentation string for this property. It will be automatically
@@ -88,17 +86,24 @@ class Property(PropertyDescriptorFactory):
             Whether attributes created from this property are read-only.
             (default: False)
 
-    '''
+    """
 
     # This class attribute is controlled by external helper API for validation
     _should_validate = True
 
-    def __init__(self, default=None, help=None, serialized=True, readonly=False):
-        # This is how the descriptor is created in the class declaration.
-        self._serialized = False if readonly else serialized
+    def __init__(self, default=Intrinsic, help=None, serialized=None, readonly=False):
+        default = default if default is not Intrinsic else Undefined
+
+        if serialized is None:
+            self._serialized = False if readonly and default is Undefined else True
+        else:
+            self._serialized = serialized
+
         self._readonly = readonly
         self._default = default
+        self._help = help
         self.__doc__ = help
+
         self.alternatives = []
         self.assertions = []
 
@@ -107,27 +112,31 @@ class Property(PropertyDescriptorFactory):
 
     @classmethod
     def _sphinx_prop_link(cls):
-        ''' Generate a sphinx :class: link to this property.
+        """ Generate a sphinx :class: link to this property.
 
-        '''
-        return ":class:`~bokeh.core.properties.%s` " % cls.__name__
+        """
+        # (double) escaped space at the end is to appease Sphinx
+        # https://www.sphinx-doc.org/en/master/usage/restructuredtext/basics.html#gotchas
+        return f":class:`~bokeh.core.properties.{cls.__name__}`\\ "
 
     @staticmethod
     def _sphinx_model_link(name):
-        ''' Generate a sphinx :class: link to given named model.
+        """ Generate a sphinx :class: link to given named model.
 
-        '''
-        return ":class:`~%s` " % name
+        """
+        # (double) escaped space at the end is to appease Sphinx
+        # https://www.sphinx-doc.org/en/master/usage/restructuredtext/basics.html#gotchas
+        return f":class:`~{name}`\\ "
 
     def _sphinx_type(self):
-        ''' Generate a Sphinx-style reference to this type for documentation
+        """ Generate a Sphinx-style reference to this type for documentation
         automation purposes.
 
-        '''
+        """
         return self._sphinx_prop_link()
 
     def make_descriptors(self, base_name):
-        ''' Return a list of ``BasicPropertyDescriptor`` instances to install
+        """ Return a list of ``BasicPropertyDescriptor`` instances to install
         on a class, in order to delegate attribute access to this property.
 
         Args:
@@ -138,42 +147,42 @@ class Property(PropertyDescriptorFactory):
 
         The descriptors returned are collected by the ``MetaHasProps``
         metaclass and added to ``HasProps`` subclasses during class creation.
-        '''
+        """
         return [ BasicPropertyDescriptor(base_name, self) ]
 
     def _may_have_unstable_default(self):
-        ''' False if we have a default that is immutable, and will be the
+        """ False if we have a default that is immutable, and will be the
         same every time (some defaults are generated on demand by a function
         to be called).
 
-        '''
+        """
         return isinstance(self._default, types.FunctionType)
 
     @classmethod
     def _copy_default(cls, default):
-        ''' Return a copy of the default, or a new value if the default
+        """ Return a copy of the default, or a new value if the default
         is specified by a function.
 
-        '''
+        """
         if not isinstance(default, types.FunctionType):
             return copy(default)
         else:
             return default()
 
     def _raw_default(self):
-        ''' Return the untransformed default value.
+        """ Return the untransformed default value.
 
         The raw_default() needs to be validated and transformed by
         prepare_value() before use, and may also be replaced later by
         subclass overrides or by themes.
 
-        '''
+        """
         return self._copy_default(self._default)
 
     def themed_default(self, cls, name, theme_overrides):
-        ''' The default, transformed by prepare_value() and the theme overrides.
+        """ The default, transformed by prepare_value() and the theme overrides.
 
-        '''
+        """
         overrides = theme_overrides
         if overrides is None or name not in overrides:
             overrides = cls._overridden_defaults()
@@ -186,25 +195,25 @@ class Property(PropertyDescriptorFactory):
 
     @property
     def serialized(self):
-        ''' Whether the property should be serialized when serializing an object.
+        """ Whether the property should be serialized when serializing an object.
 
         This would be False for a "virtual" or "convenience" property that duplicates
         information already available in other properties, for example.
-        '''
+        """
         return self._serialized
 
     @property
     def readonly(self):
-        ''' Whether this property is read-only.
+        """ Whether this property is read-only.
 
         Read-only properties may only be modified by the client (i.e., by BokehJS
         in the browser).
 
-        '''
+        """
         return self._readonly
 
     def matches(self, new, old):
-        ''' Whether two parameters match values.
+        """ Whether two parameters match values.
 
         If either ``new`` or ``old`` is a NumPy array or Pandas Series or Index,
         then the result of ``np.array_equal`` will determine if the values match.
@@ -214,7 +223,7 @@ class Property(PropertyDescriptorFactory):
         Returns:
             True, if new and old match, False otherwise
 
-        '''
+        """
         if isinstance(new, np.ndarray) or isinstance(old, np.ndarray):
             return np.array_equal(new, old)
 
@@ -234,6 +243,9 @@ class Property(PropertyDescriptorFactory):
                     return False
                 return all(self.matches(new[k], old[k]) for k in new)
 
+            # FYI Numpy can erroneously raise a warning about elementwise
+            # comparison here when a timedelta is compared to another scalar.
+            # https://github.com/numpy/numpy/issues/10095
             return new == old
 
         # if the comparison fails for some reason, just punt and return no-match
@@ -241,21 +253,21 @@ class Property(PropertyDescriptorFactory):
             return False
 
     def from_json(self, json, models=None):
-        ''' Convert from JSON-compatible values into a value for this property.
+        """ Convert from JSON-compatible values into a value for this property.
 
         JSON-compatible values are: list, dict, number, string, bool, None
 
-        '''
+        """
         return json
 
     def serialize_value(self, value):
-        ''' Change the value into a JSON serializable format.
+        """ Change the value into a JSON serializable format.
 
-        '''
+        """
         return value
 
     def transform(self, value):
-        ''' Change the value into the canonical format for this property.
+        """ Change the value into the canonical format for this property.
 
         Args:
             value (obj) : the value to apply transformation to.
@@ -263,11 +275,11 @@ class Property(PropertyDescriptorFactory):
         Returns:
             obj: transformed value
 
-        '''
+        """
         return value
 
     def validate(self, value, detail=True):
-        ''' Determine whether we can set this property from this value.
+        """ Determine whether we can set this property from this value.
 
         Validation happens before transform()
 
@@ -286,11 +298,11 @@ class Property(PropertyDescriptorFactory):
         Raises:
             ValueError if the value is not valid for this property type
 
-        '''
+        """
         pass
 
     def is_valid(self, value):
-        ''' Whether the value passes validation
+        """ Whether the value passes validation
 
         Args:
             value (obj) : the value to validate against this property type
@@ -298,7 +310,7 @@ class Property(PropertyDescriptorFactory):
         Returns:
             True if valid, False otherwise
 
-        '''
+        """
         try:
             if validation_on():
                 self.validate(value, False)
@@ -307,14 +319,19 @@ class Property(PropertyDescriptorFactory):
         else:
             return True
 
-    @classmethod
-    def wrap(cls, value):
-        ''' Some property types need to wrap their values in special containers, etc.
+    def wrap(self, value):
+        """ Some property types need to wrap their values in special containers, etc.
 
-        '''
+        """
         return value
 
-    def prepare_value(self, obj_or_cls, name, value):
+    def prepare_value(self, owner: Union[HasProps, Type[HasProps]], name: str, value: Any):
+        if value is Intrinsic:
+            value = self._raw_default()
+        if value is Undefined:
+            return value
+
+        error = None
         try:
             if validation_on():
                 self.validate(value)
@@ -324,12 +341,16 @@ class Property(PropertyDescriptorFactory):
                     value = converter(value)
                     break
             else:
-                raise e
-        else:
-            value = self.transform(value)
+                error = e
 
-        if isinstance(obj_or_cls, HasProps):
-            obj = obj_or_cls
+        if error is None:
+            value = self.transform(value)
+        else:
+            obj_repr = owner if isinstance(owner, HasProps) else owner.__name__
+            raise ValueError(f"failed to validate {obj_repr}.{name}: {error}")
+
+        if isinstance(owner, HasProps):
+            obj = owner
 
             for fn, msg_or_fn in self.assertions:
                 if isinstance(fn, bool):
@@ -340,7 +361,7 @@ class Property(PropertyDescriptorFactory):
                 assert isinstance(result, bool)
 
                 if not result:
-                    if isinstance(msg_or_fn, string_types):
+                    if isinstance(msg_or_fn, str):
                         raise ValueError(msg_or_fn)
                     else:
                         msg_or_fn(obj, name, value)
@@ -352,7 +373,7 @@ class Property(PropertyDescriptorFactory):
         return False
 
     def accepts(self, tp, converter):
-        ''' Declare that other types may be converted to this property type.
+        """ Declare that other types may be converted to this property type.
 
         Args:
             tp (Property) :
@@ -366,14 +387,14 @@ class Property(PropertyDescriptorFactory):
         Returns:
             self
 
-        '''
+        """
 
         tp = ParameterizedProperty._validate_type_param(tp)
         self.alternatives.append((tp, converter))
         return self
 
     def asserts(self, fn, msg_or_fn):
-        ''' Assert that prepared values satisfy given conditions.
+        """ Assert that prepared values satisfy given conditions.
 
         Assertions are intended in enforce conditions beyond simple value
         type validation. For instance, this method can be use to assert that
@@ -393,15 +414,14 @@ class Property(PropertyDescriptorFactory):
         Returns:
             self
 
-        '''
+        """
         self.assertions.append((fn, msg_or_fn))
         return self
 
 class ParameterizedProperty(Property):
-    ''' A base class for Properties that have type parameters, e.g.
-    ``List(String)``.
+    """ A base class for Properties that have type parameters, e.g. ``List(String)``.
 
-    '''
+    """
 
     @staticmethod
     def _validate_type_param(type_param):
@@ -411,9 +431,12 @@ class ParameterizedProperty(Property):
             else:
                 type_param = type_param.__name__
         elif isinstance(type_param, Property):
+            if type_param._help is not None:
+                raise ValueError("setting 'help' on type parameters doesn't make sense")
+
             return type_param
 
-        raise ValueError("expected a Propertyas type parameter, got %s" % type_param)
+        raise ValueError(f"expected a Property as type parameter, got {type_param}")
 
     @property
     def type_params(self):
@@ -423,8 +446,42 @@ class ParameterizedProperty(Property):
     def has_ref(self):
         return any(type_param.has_ref for type_param in self.type_params)
 
+class SingleParameterizedProperty(ParameterizedProperty):
+    """ A parameterized property with a single type parameter. """
+
+    def __init__(self, type_param, *, default=Intrinsic, help=None, serialized=None, readonly=False):
+        self.type_param = self._validate_type_param(type_param)
+        default = default if default is not Intrinsic else self.type_param._raw_default()
+        super().__init__(default=default, help=help, serialized=serialized, readonly=readonly)
+
+    @property
+    def type_params(self):
+        return [self.type_param]
+
+    def __str__(self):
+        return f"{self.__class__.__name__}({self.type_param})"
+
+    def _sphinx_type(self):
+        return f"{self._sphinx_prop_link()}({self.type_param._sphinx_type()})"
+
+    def validate(self, value: Any, detail: bool = True) -> None:
+        super().validate(value, detail=detail)
+        self.type_param.validate(value, detail=detail)
+
+    def from_json(self, json, models=None):
+        return self.type_param.from_json(json, models=models)
+
+    def transform(self, value):
+        return self.type_param.transform(value)
+
+    def wrap(self, value):
+        return self.type_param.wrap(value)
+
+    def _may_have_unstable_default(self):
+        return self.type_param._may_have_unstable_default()
+
 class PrimitiveProperty(Property):
-    ''' A base class for simple property types.
+    """ A base class for simple property types.
 
     Subclasses should define a class attribute ``_underlying_type`` that is
     a tuple of acceptable type values for the property.
@@ -438,45 +495,49 @@ class PrimitiveProperty(Property):
             class Float(PrimitiveProperty):
                 _underlying_type = (numbers.Real,)
 
-    '''
+    """
 
     _underlying_type = None
 
     def validate(self, value, detail=True):
-        super(PrimitiveProperty, self).validate(value, detail)
+        super().validate(value, detail)
 
-        if not (value is None or isinstance(value, self._underlying_type)):
-            msg = "" if not detail else "expected a value of type %s, got %s of type %s" % (
-                nice_join([ cls.__name__ for cls in self._underlying_type ]), value, type(value).__name__
-            )
-            raise ValueError(msg)
+        if isinstance(value, self._underlying_type):
+            return
+
+        if not detail:
+            raise ValueError("")
+
+        expected_type = nice_join([ cls.__name__ for cls in self._underlying_type ])
+        msg = f"expected a value of type {expected_type}, got {value} of type {type(value).__name__}"
+        raise ValueError(msg)
 
     def from_json(self, json, models=None):
-        if json is None or isinstance(json, self._underlying_type):
+        if isinstance(json, self._underlying_type):
             return json
-        else:
-            expected = nice_join([ cls.__name__ for cls in self._underlying_type ])
-            raise DeserializationError("%s expected %s, got %s" % (self, expected, json))
+        expected_type = nice_join([ cls.__name__ for cls in self._underlying_type ])
+        msg = f"{self} expected {expected_type}, got {json} of type {type(json).__name__}"
+        raise DeserializationError(msg)
 
     def _sphinx_type(self):
         return self._sphinx_prop_link()
 
 class ContainerProperty(ParameterizedProperty):
-    ''' A base class for Container-like type properties.
+    """ A base class for Container-like type properties.
 
-    '''
+    """
 
     def _may_have_unstable_default(self):
         # all containers are mutable, so the default can be modified
         return True
 
 def validation_on():
-    ''' Check if property validation is currently active
+    """ Check if property validation is currently active
 
     Returns:
         bool
 
-    '''
+    """
     return Property._should_validate
 
 #-----------------------------------------------------------------------------
