@@ -1,12 +1,12 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
 #-----------------------------------------------------------------------------
-''' Provide date and time related properties
+""" Provide date and time related properties
 
-'''
+"""
 
 #-----------------------------------------------------------------------------
 # Boilerplate
@@ -32,6 +32,7 @@ from ...util.serialization import (
 )
 from .bases import Property
 from .primitive import bokeh_integer_types
+from .singletons import Undefined
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -48,9 +49,9 @@ __all__ = (
 #-----------------------------------------------------------------------------
 
 class Date(Property):
-    ''' Accept Date (but not DateTime) values.
+    """ Accept Date (but not DateTime) values.
 
-    '''
+    """
     def transform(self, value):
         value = super().transform(value)
 
@@ -64,9 +65,6 @@ class Date(Property):
     def validate(self, value, detail=True):
         super().validate(value, detail)
 
-        if value is None:
-            return
-
         # datetime.datetime is datetime.date, exclude manually up front
         if isinstance(value, datetime.datetime):
             msg = "" if not detail else "Expected a date value, got a datetime.datetime"
@@ -82,11 +80,11 @@ class Date(Property):
             raise ValueError(msg)
 
 class Datetime(Property):
-    ''' Accept Datetime values.
+    """ Accept Datetime values.
 
-    '''
+    """
 
-    def __init__(self, default=None, help=None):
+    def __init__(self, default=Undefined, help=None):
         super().__init__(default=default, help=help)
 
     def transform(self, value):
@@ -104,9 +102,6 @@ class Datetime(Property):
     def validate(self, value, detail=True):
         super().validate(value, detail)
 
-        if value is None:
-            return
-
         if is_datetime_type(value):
             return
 
@@ -127,9 +122,9 @@ class Datetime(Property):
         raise ValueError(msg)
 
     def serialize_value(self, value):
-        ''' Change the value into a JSON serializable format.
+        """ Change the value into a JSON serializable format.
 
-        '''
+        """
         if isinstance(value, datetime.date):
             value = convert_date_to_datetime(value)
         return value
@@ -139,9 +134,9 @@ class Datetime(Property):
         return isinstance(value, (float,) + bokeh_integer_types) and not isinstance(value, bool)
 
 class TimeDelta(Property):
-    ''' Accept TimeDelta values.
+    """ Accept TimeDelta values.
 
-    '''
+    """
 
     def __init__(self, default=datetime.timedelta(), help=None):
         super().__init__(default=default, help=help)
