@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -8,17 +8,16 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
 # Standard library imports
+from os import PathLike
 from os.path import join
 from subprocess import Popen
-
-# External imports
+from typing import AnyStr, List, Optional
 
 # Bokeh imports
 from . import __version__
@@ -35,9 +34,8 @@ __all__ = ["init", "build"]
 # General API
 #-----------------------------------------------------------------------------
 
-# TODO: def init(base_dir: pathlib.Path, *, interactive: bool = False,
-#                development: bool = False, debug: bool = False) -> bool
-def init(base_dir, interactive=False, bokehjs_version=None, debug=False):
+def init(base_dir: "PathLike[AnyStr]", *, interactive: bool = False,
+         bokehjs_version: Optional[str] = None, debug: bool = False) -> bool:
     """
     Initialize a directory as a new bokeh extension.
 
@@ -62,8 +60,8 @@ def init(base_dir, interactive=False, bokehjs_version=None, debug=False):
     proc = _run_command("init", base_dir, args, debug)
     return proc.returncode == 0
 
-# TODO: def init(base_dir: pathlib.Path, *, rebuild: bool = False, debug: bool = False) -> bool
-def build(base_dir, rebuild=False, debug=False):
+
+def build(base_dir: "PathLike[AnyStr]", *, rebuild: bool = False, debug: bool = False) -> bool:
     """
     Build a bokeh extension in the given directory.
 
@@ -92,7 +90,7 @@ def build(base_dir, rebuild=False, debug=False):
 # Private API
 #-----------------------------------------------------------------------------
 
-def _run_command(command, base_dir, args, debug=False):
+def _run_command(command: str, base_dir: "PathLike[AnyStr]", args: List[str], debug: bool = False) -> "Popen[bytes]":
     bokehjs_dir = settings.bokehjsdir()
 
     if debug:
