@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,30 +11,49 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
-from ..core.enums import Align, SizingMode, SizingPolicy, Location
+from ..core.enums import Align, Location, SizingMode, SizingPolicy
 from ..core.has_props import abstract
-from ..core.properties import (Bool, Auto, Enum, Int, NonNegativeInt, Float,
-    Instance, List, Seq, Tuple, Dict, String, Either, Struct, Color)
-from ..core.validation import warning, error
-from ..core.validation.warnings import (BOTH_CHILD_AND_ROOT, EMPTY_LAYOUT,
-    FIXED_SIZING_MODE, FIXED_WIDTH_POLICY, FIXED_HEIGHT_POLICY)
-from ..core.validation.errors import MIN_PREFERRED_MAX_WIDTH, MIN_PREFERRED_MAX_HEIGHT
+from ..core.properties import (
+    Auto,
+    Bool,
+    Color,
+    Dict,
+    Either,
+    Enum,
+    Float,
+    Instance,
+    Int,
+    List,
+    NonNegativeInt,
+    Null,
+    Nullable,
+    Seq,
+    String,
+    Struct,
+    Tuple,
+)
+from ..core.validation import error, warning
+from ..core.validation.errors import (
+    MIN_PREFERRED_MAX_HEIGHT,
+    MIN_PREFERRED_MAX_WIDTH,
+    REPEATED_LAYOUT_CHILD,
+)
+from ..core.validation.warnings import (
+    BOTH_CHILD_AND_ROOT,
+    EMPTY_LAYOUT,
+    FIXED_HEIGHT_POLICY,
+    FIXED_SIZING_MODE,
+    FIXED_WIDTH_POLICY,
+)
 from ..model import Model
-from .callbacks import Callback
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -73,36 +92,39 @@ class LayoutDOM(Model):
     Whether the component will be visible and a part of a layout.
     """)
 
-    width = NonNegativeInt(default=None, help="""
+    width = Nullable(NonNegativeInt, help="""
     The width of the component (in pixels).
 
     This can be either fixed or preferred width, depending on width sizing policy.
     """)
 
-    height = NonNegativeInt(default=None, help="""
+    height = Nullable(NonNegativeInt, help="""
     The height of the component (in pixels).
 
     This can be either fixed or preferred height, depending on height sizing policy.
     """)
 
-    min_width = NonNegativeInt(default=None, help="""
+    min_width = Nullable(NonNegativeInt, help="""
     Minimal width of the component (in pixels) if width is adjustable.
     """)
 
-    min_height = NonNegativeInt(default=None, help="""
+    min_height = Nullable(NonNegativeInt, help="""
     Minimal height of the component (in pixels) if height is adjustable.
     """)
 
-    max_width = NonNegativeInt(default=None, help="""
-    Minimal width of the component (in pixels) if width is adjustable.
+    max_width = Nullable(NonNegativeInt, help="""
+    Maximal width of the component (in pixels) if width is adjustable.
     """)
 
-    max_height = NonNegativeInt(default=None, help="""
-    Minimal height of the component (in pixels) if height is adjustable.
+    max_height = Nullable(NonNegativeInt, help="""
+    Maximal height of the component (in pixels) if height is adjustable.
     """)
 
-    margin = Tuple(Int, Int, Int, Int, default=(0, 0, 0, 0), help="""
+    margin = Nullable(Tuple(Int, Int, Int, Int), default=(0, 0, 0, 0), help="""
     Allows to create additional space around the component.
+    The values in the tuple are ordered as follows - Margin-Top, Margin-Right, Margin-Bottom and Margin-Left,
+    similar to CSS standards.
+    Negative margin values may be used to shrink the space from any direction.
     """).accepts(Tuple(Int, Int), lambda v_h: (v_h[0], v_h[1], v_h[0], v_h[1])) \
         .accepts(Int, lambda m: (m, m, m, m))
 
@@ -170,7 +192,7 @@ class LayoutDOM(Model):
 
     """)
 
-    aspect_ratio = Either(Enum("auto"), Float, default=None, help="""
+    aspect_ratio = Either(Null, Auto, Float, help="""
     Describes the proportional relationship between component's width and height.
 
     This works if any of component's dimensions are flexible in size. If set to
@@ -180,7 +202,7 @@ class LayoutDOM(Model):
 
     """)
 
-    sizing_mode = Enum(SizingMode, default=None, help="""
+    sizing_mode = Nullable(Enum(SizingMode), help="""
     How the component should size itself.
 
     This is a high-level setting for maintaining width and height of the component. To
@@ -230,7 +252,7 @@ class LayoutDOM(Model):
     grid track align).
     """)
 
-    background = Color(default=None, help="""
+    background = Nullable(Color, help="""
     Background color of the component.
     """)
 
@@ -339,6 +361,12 @@ class GridBox(LayoutDOM):
     respectively.
     """)
 
+    @error(REPEATED_LAYOUT_CHILD)
+    def _check_repeated_layout_children(self):
+        children = [ child[0] for child in self.children ]
+        if len(children) != len(set(children)):
+            return str(self)
+
 @abstract
 class Box(LayoutDOM):
     ''' Abstract base class for Row and Column. Do not use directly.
@@ -352,7 +380,7 @@ class Box(LayoutDOM):
         elif len(args) > 0:
             kwargs["children"] = list(args)
 
-        super(Box, self).__init__(**kwargs)
+        super().__init__(**kwargs)
 
     @warning(EMPTY_LAYOUT)
     def _check_empty_layout(self):
@@ -371,6 +399,11 @@ class Box(LayoutDOM):
         else:
             return None
 
+    @error(REPEATED_LAYOUT_CHILD)
+    def _check_repeated_layout_children(self):
+        if len(self.children) != len(set(self.children)):
+            return str(self)
+
     children = List(Instance(LayoutDOM), help="""
     The list of children, which can be other components including plots, rows, columns, and widgets.
     """)
@@ -412,11 +445,6 @@ class Column(Box):
 
     """)
 
-class WidgetBox(Column):
-    ''' Create a column of bokeh widgets with predefined styling.
-
-    '''
-
 class Panel(Model):
     ''' A single-widget container with title bar and controls.
 
@@ -456,10 +484,6 @@ class Tabs(LayoutDOM):
     The index of the active tab.
     """)
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever the button is activated.
-    """)
-
 #-----------------------------------------------------------------------------
 # Dev API
 #-----------------------------------------------------------------------------
@@ -471,3 +495,15 @@ class Tabs(LayoutDOM):
 #-----------------------------------------------------------------------------
 # Code
 #-----------------------------------------------------------------------------
+
+# TODO (bev) deprecation: 3.0
+class WidgetBox(Column):
+    ''' Create a column of bokeh widgets with predefined styling.
+
+    WidgetBox is DEPRECATED and will beremoved in Bokeh 3.0, use 'Column' instead.
+
+    '''
+    def __init__(self, *args, **kw):
+        super().__init__(*args, **kw)
+        from ..util.deprecation import deprecated
+        deprecated("'WidgetBox' is deprecated and will be removed in Bokeh 3.0, use 'bokeh.models.Column' instead")
