@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -8,19 +8,13 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from ..core.has_props import abstract
 from ..core.properties import Dict, Int, Seq, String
@@ -55,7 +49,7 @@ class Selection(Model):
 
     indices = Seq(Int, default=[], help="""
     The "scatter" level indices included in a selection. For example, for a
-    selection on a ``Circle`` glyph, this list records the indices of whicn
+    selection on a ``Circle`` glyph, this list records the indices of which
     individual circles are selected.
 
     For "multi" glyphs such as ``Patches``, ``MultiLine``, ``MultiPolygons``,
