@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,25 +11,16 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from ...core.has_props import abstract
-from ...core.properties import Bool, Instance, Int, List, String
-
-from ..callbacks import Callback
-
+from ...core.properties import Bool, Int, List, Nullable, String
 from .buttons import ButtonLike
 from .widget import Widget
 
@@ -78,10 +69,6 @@ class AbstractGroup(Widget):
         ''' Set up a handler for button check/radio box clicks including the selected indices. '''
         self.js_on_change('active', handler)
 
-    callback = Instance(Callback, help="""
-    A callback to run in the browser whenever a button group is manipulated.
-    """)
-
 @abstract
 class ButtonGroup(AbstractGroup, ButtonLike):
     ''' Abstract base class for groups with items rendered as buttons.
@@ -118,7 +105,7 @@ class RadioGroup(Group):
 
     '''
 
-    active = Int(None, help="""
+    active = Nullable(Int, help="""
     The index of the selected radio box, or ``None`` if nothing is
     selected.
     """)
@@ -137,7 +124,7 @@ class RadioButtonGroup(ButtonGroup):
 
     '''
 
-    active = Int(None, help="""
+    active = Nullable(Int, help="""
     The index of the selected radio box, or ``None`` if nothing is
     selected.
     """)
