@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -17,23 +17,16 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-
 # Bokeh imports
 from ...core.has_props import abstract
-from ...core.properties import Any, Bool, String, Dict
-
+from ...core.properties import Any, Bool, Dict, String
 from .widget import Widget
 
 #-----------------------------------------------------------------------------
@@ -60,7 +53,16 @@ class Markup(Widget):
     '''
 
     text = String(default="", help="""
-    The contents of the widget.
+    The text or HTML contents of the widget.
+
+    .. note::
+        If the HTML content contains elements which size depends on
+        on external, asynchronously loaded resources, the size of
+        the widget may be computed incorrectly. This is in particular
+        an issue with images (``<img>``). To remedy this problem, one
+        either has to set explicit dimensions using CSS properties,
+        HTML attributes or model's ``width`` and ``height`` properties,
+        or inline images' contents using data URIs.
     """)
 
     style = Dict(String, Any, default={}, help="""
