@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -37,11 +37,13 @@ from ...core.properties import (
     Instance,
     Int,
     List,
+    NonNullable,
+    Nullable,
     Override,
     String,
 )
 from ...model import Model
-from ..sources import CDSView, DataSource
+from ..sources import CDSView, ColumnDataSource, DataSource
 from .widget import Widget
 
 #-----------------------------------------------------------------------------
@@ -119,9 +121,8 @@ class StringFormatter(CellFormatter):
     An optional text align, i.e. left, center or right.
     """)
 
-    text_color = Color(help="""
-    An optional text color. See :class:`bokeh.core.properties.Color` for
-    details.
+    text_color = Nullable(Color, help="""
+    An optional text color.
     """)
 
 class ScientificFormatter(StringFormatter):
@@ -129,7 +130,7 @@ class ScientificFormatter(StringFormatter):
     using scientific notation when appropriate by default.
     '''
 
-    nan_format = String(help="""
+    nan_format = Nullable(String, help="""
     Formatting to apply to NaN and None values (falls back to scientific formatting if not set).
     """)
 
@@ -232,7 +233,7 @@ class NumberFormatter(StringFormatter):
     The language to use for formatting language-specific features (e.g. thousands separator).
     """)
 
-    nan_format = String(help="""
+    nan_format = Nullable(String, help="""
     Formatting to apply to NaN and None values (falls back to Numbro formatting if not set).
     """)
 
@@ -459,7 +460,7 @@ class DateFormatter(StringFormatter):
 
     """)
 
-    nan_format = String(help="""
+    nan_format = Nullable(String, help="""
     Formatting to apply to NaN and None values (falls back to regular date formatting if not set).
     """)
 
@@ -582,11 +583,11 @@ class TableColumn(Model):
 
     '''
 
-    field = String(help="""
+    field = NonNullable(String, help="""
     The name of the field mapping to a column in the data source.
     """)
 
-    title = String(help="""
+    title = Nullable(String, help="""
     The title of this column. If not set, column's data field is
     used instead.
     """)
@@ -621,7 +622,7 @@ class TableWidget(Widget):
 
     '''
 
-    source = Instance(DataSource, help="""
+    source = Instance(DataSource, default=lambda: ColumnDataSource(), help="""
     The source of data for the widget.
     """)
 
@@ -637,7 +638,7 @@ class TableWidget(Widget):
             self.view = CDSView(source=self.source)
 
 class DataTable(TableWidget):
-    ''' Two dimensional grid for visualisation and editing large amounts
+    ''' Two-dimensional grid for visualization and editing large amounts
     of data.
 
     '''
@@ -646,7 +647,7 @@ class DataTable(TableWidget):
     Describes the column autosizing mode with one of the following options:
 
     ``"fit_columns"``
-        Compute columns widths based on cell contents but ensure the
+        Compute column widths based on cell contents but ensure the
         table fits into the available viewport. This results in no
         horizontal scrollbar showing up, but data can get unreadable
         if there is not enough space available.
@@ -674,19 +675,22 @@ class DataTable(TableWidget):
     The list of child column widgets.
     """)
 
-    fit_columns = Bool(help="""
-    Whether columns should be fit to the available width. This results in no
-    horizontal scrollbar showing up, but data can get unreadable if there is
-    not enough space available. If set to ``True``, columns' width is
+    fit_columns = Nullable(Bool, help="""
+    **This is a legacy parameter.** For new development, use the
+    ``autosize_mode`` parameter.
+
+    Whether columns should be fit to the available width. This results in
+    no horizontal scrollbar showing up, but data can get unreadable if there
+    is not enough space available. If set to ``True``, each column's width is
     understood as maximum width.
     """)
 
-    frozen_columns = Int(help="""
+    frozen_columns = Nullable(Int, help="""
     Integer indicating the number of columns to freeze. If set the first N
     columns will be frozen which prevents them from scrolling out of frame.
     """)
 
-    frozen_rows = Int(help="""
+    frozen_rows = Nullable(Int, help="""
     Integer indicating the number of rows to freeze. If set the first N
     rows will be frozen which prevents them from scrolling out of frame,
     if set to a negative value last N rows will be frozen.
@@ -718,7 +722,7 @@ class DataTable(TableWidget):
     enabled) or using Shift + click on rows.
     """)
 
-    index_position = Int(0, help="""
+    index_position = Nullable(Int, default=0, help="""
     Where among the list of columns to insert a column displaying the row
     index. Negative indices are supported, and specify an index position
     from the end of the list of columns (i.e. standard Python behaviour).
