@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -116,7 +116,7 @@ This module contains the following sets of palettes:
 * All `ColorBrewer`_ palettes
 * Categorical `D3`_ palettes
 * The `Matplotlib`_ palettes Magma, Inferno, Plasma, and Viridis
-* A 'Bokeh'_ palette comprised of the Bokeh shutter logo colors
+* A Bokeh palette comprised of the Bokeh shutter logo colors
 * Palettes designed for color-deficient usability
 
 Additionally, you can also use any of the 256-color perceptually uniform
@@ -1099,6 +1099,7 @@ Colorblind4 = Colorblind8[:4]
 Colorblind3 = Colorblind8[:3]
 
 # Bokeh palette created from colors of shutter logo
+Bokeh8 = ('#EC1557', '#F05223', '#F6A91B', '#A5CD39', '#20B254', '#00AAAE', '#4998D3', '#892889')
 Bokeh7 = ('#EC1557', '#F05223', '#F6A91B', '#A5CD39', '#20B254', '#00AAAE', '#892889')
 Bokeh6 = Bokeh7[:6]
 Bokeh5 = Bokeh7[:5]
@@ -1162,7 +1163,7 @@ Category20c = { 3:  Category20c_3,  4:  Category20c_4,  5:  Category20c_5,  6:
                 13: Category20c_13, 14: Category20c_14, 15: Category20c_15, 16: Category20c_16, 17: Category20c_17,
                 18: Category20c_18, 19: Category20c_19, 20: Category20c_20 }
 Colorblind  = { 3: Colorblind3, 4: Colorblind4, 5: Colorblind5, 6: Colorblind6, 7: Colorblind7, 8: Colorblind8 }
-Bokeh = { 3: Bokeh3, 4: Bokeh4, 5: Bokeh5, 6: Bokeh6, 7: Bokeh7 }
+Bokeh = { 3: Bokeh3, 4: Bokeh4, 5: Bokeh5, 6: Bokeh6, 7: Bokeh7, 8: Bokeh8 }
 
 brewer = {
     "YlGn"     : YlGn,
@@ -1246,7 +1247,7 @@ del small_palettes["Cividis"][256]
 del small_palettes["Turbo"][256]
 
 def linear_palette(palette: Palette, n: int) -> Palette:
-    ''' Generate a new palette as a subset of a given palette.
+    """ Generate a new palette as a subset of a given palette.
 
     Given an input ``palette``, take ``n`` colors from it by dividing its
     length into ``n`` (approximately) evenly spaced indices.
@@ -1260,15 +1261,15 @@ def linear_palette(palette: Palette, n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if ``n > len(palette)``
+        ValueError if n > len(palette)
 
-    '''
+    """
     if n > len(palette):
-        raise ValueError("Requested %(r)s colors, function can only return colors up to the base palette's length (%(l)s)" % dict(r=n, l=len(palette)))
+        raise ValueError(f"Requested {n} colors, function can only return colors up to the base palette's length ({len(palette)})")
     return tuple( palette[int(math.floor(i))] for i in np.linspace(0, len(palette)-1, num=n) )
 
 def diverging_palette(palette1: Palette, palette2: Palette, n: int, midpoint: float = 0.5) -> Palette:
-    ''' Generate a new palette by combining exactly two input palettes.
+    """ Generate a new palette by combining exactly two input palettes.
 
     Given an input ``palette1`` and ``palette2``, take a combined ``n`` colors,
     and combine input palettes at the relative ``midpoint``.
@@ -1296,8 +1297,8 @@ def diverging_palette(palette1: Palette, palette2: Palette, n: int, midpoint: fl
             seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the possible combined length the input palettes
-    '''
+        ValueError if n is greater than the possible combined length the input palettes
+    """
 
     # flip palette2 so that perceptually light colors are joined
     palette2 = palette2[::-1]
@@ -1310,7 +1311,7 @@ def diverging_palette(palette1: Palette, palette2: Palette, n: int, midpoint: fl
     return linear_palette(palette1, n1) + linear_palette(palette2, n2)
 
 def magma(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Magma palette.
+    """ Generate a palette of colors or from the Magma palette.
 
     The full Magma palette that serves as input for deriving new palettes
     has 256 colors, and looks like:
@@ -1324,7 +1325,7 @@ def magma(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1335,11 +1336,11 @@ def magma(n: int) -> Palette:
 
     The resulting palette looks like: :bokeh-palette:`magma(6)`
 
-    '''
+    """
     return linear_palette(Magma256, n)
 
 def inferno(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Inferno palette.
+    """ Generate a palette of colors or from the Inferno palette.
 
     The full Inferno palette that serves as input for deriving new palettes
     has 256 colors, and looks like:
@@ -1353,7 +1354,7 @@ def inferno(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1364,11 +1365,11 @@ def inferno(n: int) -> Palette:
 
     The resulting palette looks like: :bokeh-palette:`inferno(6)`
 
-    '''
+    """
     return linear_palette(Inferno256, n)
 
 def plasma(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Plasma palette.
+    """ Generate a palette of colors or from the Plasma palette.
 
     The full Plasma palette that serves as input for deriving new palettes
     has 256 colors, and looks like:
@@ -1382,7 +1383,7 @@ def plasma(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1393,11 +1394,11 @@ def plasma(n: int) -> Palette:
 
     The resulting palette looks like: :bokeh-palette:`plasma(6)`
 
-    '''
+    """
     return linear_palette(Plasma256, n)
 
 def viridis(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Viridis palette.
+    """ Generate a palette of colors or from the Viridis palette.
 
     The full Viridis palette that serves as input for deriving new palettes
     has 256 colors, and looks like:
@@ -1411,7 +1412,7 @@ def viridis(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1422,11 +1423,11 @@ def viridis(n: int) -> Palette:
 
     The resulting palette looks like: :bokeh-palette:`viridis(6)`
 
-    '''
+    """
     return linear_palette(Viridis256, n)
 
 def cividis(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Cividis palette.
+    """ Generate a palette of colors or from the Cividis palette.
 
     The full Cividis palette that serves as input for deriving new palettes
     has 256 colors, and looks like:
@@ -1440,7 +1441,7 @@ def cividis(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1451,11 +1452,11 @@ def cividis(n: int) -> Palette:
 
     The resulting palette looks like: :bokeh-palette:`cividis(6)`
 
-    '''
+    """
     return linear_palette(Cividis256, n)
 
 def turbo(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Turbo palette.
+    """ Generate a palette of colors or from the Turbo palette.
 
     Turbo is described here:
 
@@ -1473,7 +1474,7 @@ def turbo(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1484,11 +1485,11 @@ def turbo(n: int) -> Palette:
 
     The resulting palette looks like: :bokeh-palette:`turbo(6)`
 
-    '''
+    """
     return linear_palette(Turbo256, n)
 
 def grey(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Greys palette.
+    """ Generate a palette of colors or from the Greys palette.
 
     The full Greys palette that serves as input for deriving new palettes
     has 256 colors, and looks like:
@@ -1502,7 +1503,7 @@ def grey(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1516,11 +1517,11 @@ def grey(n: int) -> Palette:
     .. note::
         This function also has the alternate spelling ``gray``
 
-    '''
+    """
     return linear_palette(Greys256, n)
 
 def gray(n: int) -> Palette:
-    ''' Generate a palette of colors or from the Greys palette.
+    """ Generate a palette of colors or from the Greys palette.
 
     The full Greys palette that serves as input for deriving new palettes
     has 256 colors, and looks like:
@@ -1534,7 +1535,7 @@ def gray(n: int) -> Palette:
         seq[str] : a sequence of hex RGB color strings
 
     Raises:
-        ``ValueError`` if n is greater than the base palette length of 256
+        ValueError if n is greater than the base palette length of 256
 
     Examples:
 
@@ -1548,7 +1549,7 @@ def gray(n: int) -> Palette:
     .. note::
         This function also has the alternate spelling ``grey``
 
-    '''
+    """
     return linear_palette(Greys256, n)
 
 #-----------------------------------------------------------------------------
