@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,27 +11,23 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
 # External imports
 from tornado import gen
 
 # Bokeh imports
-from .session import ServerSession
-
 from ..application.application import ServerContext, SessionContext
 from ..document import Document
 from ..protocol.exceptions import ProtocolError
-from ..util.tornado import _CallbackGroup, yield_for_all_futures
+from ..util.token import get_token_payload
+from ..util.tornado import _CallbackGroup
+from .session import ServerSession
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -93,22 +89,21 @@ class BokehSessionContext(SessionContext):
         self._document = document
         self._session = None
         self._logout_url = logout_url
-        super(BokehSessionContext, self).__init__(server_context,
-                                                  session_id)
+        super().__init__(server_context, session_id)
         # request arguments used to instantiate this session
         self._request = None
+        self._token = None
 
     def _set_session(self, session):
         self._session = session
 
-    @gen.coroutine
-    def with_locked_document(self, func):
+    async def with_locked_document(self, func):
         if self._session is None:
             # this means we are in on_session_created, so no locking yet,
             # we have exclusive access
-            yield yield_for_all_futures(func(self._document))
+            await func(self._document)
         else:
-            self._session.with_document_locked(func, self._document)
+            await self._session.with_document_locked(func, self._document)
 
     @property
     def destroyed(self):
@@ -126,12 +121,16 @@ class BokehSessionContext(SessionContext):
     def request(self):
         return self._request
 
+    @property
+    def token_payload(self):
+        return get_token_payload(self._token)
+
     @property
     def session(self):
         return self._session
 
 
-class ApplicationContext(object):
+class ApplicationContext:
     ''' Server-side holder for ``bokeh.application.Application`` plus any associated data.
         This holds data that's global to all sessions, while ``ServerSession`` holds
         data specific to an "instance" of the application.
@@ -171,26 +170,19 @@ class ApplicationContext(object):
 
     def run_load_hook(self):
         try:
-            result = self._application.on_server_loaded(self.server_context)
-            if isinstance(result, gen.Future):
-                log.error("on_server_loaded returned a Future; this doesn't make sense "
-                          "because we run this hook before starting the IO loop.")
+            self._application.on_server_loaded(self.server_context)
         except Exception as e:
             log.error("Error in server loaded hook %r", e, exc_info=True)
 
     def run_unload_hook(self):
         try:
-            result = self._application.on_server_unloaded(self.server_context)
-            if isinstance(result, gen.Future):
-                log.error("on_server_unloaded returned a Future; this doesn't make sense "
-                          "because we stop the IO loop right away after calling on_server_unloaded.")
+            self._application.on_server_unloaded(self.server_context)
         except Exception as e:
             log.error("Error in server unloaded hook %r", e, exc_info=True)
 
         self.server_context._remove_all_callbacks()
 
-    @gen.coroutine
-    def create_session_if_needed(self, session_id, request=None):
+    async def create_session_if_needed(self, session_id, request=None, token=None):
         # this is because empty session_ids would be "falsey" and
         # potentially open up a way for clients to confuse us
         if len(session_id) == 0:
@@ -206,22 +198,26 @@ class ApplicationContext(object):
                                                   self.server_context,
                                                   doc,
                                                   logout_url=self._logout_url)
-            # using private attr so users only have access to a read-only property
-            session_context._request = _RequestProxy(request)
+            if request is not None:
+                payload = get_token_payload(token) if token else {}
+                # using private attr so users only have access to a read-only property
+                session_context._request = _RequestProxy(request,
+                                                         cookies=payload.get('cookies'),
+                                                         headers=payload.get('headers'))
+            session_context._token = token
 
             # expose the session context to the document
             # use the _attribute to set the public property .session_context
             doc._session_context = session_context
 
-
             try:
-                yield yield_for_all_futures(self._application.on_session_created(session_context))
+                await self._application.on_session_created(session_context)
             except Exception as e:
                 log.error("Failed to run session creation hooks %r", e, exc_info=True)
 
             self._application.initialize_document(doc)
 
-            session = ServerSession(session_id, doc, io_loop=self._loop)
+            session = ServerSession(session_id, doc, io_loop=self._loop, token=token)
             del self._pending_sessions[session_id]
             self._sessions[session_id] = session
             session_context._set_session(session)
@@ -233,11 +229,11 @@ class ApplicationContext(object):
         if session_id in self._pending_sessions:
             # another create_session_if_needed is working on
             # creating this session
-            session = yield self._pending_sessions[session_id]
+            session = await self._pending_sessions[session_id]
         else:
             session = self._sessions[session_id]
 
-        raise gen.Return(session)
+        return session
 
     def get_session(self, session_id):
         if session_id in self._sessions:
@@ -246,8 +242,7 @@ class ApplicationContext(object):
         else:
             raise ProtocolError("No such session " + session_id)
 
-    @gen.coroutine
-    def _discard_session(self, session, should_discard):
+    async def _discard_session(self, session, should_discard):
         if session.connection_count > 0:
             raise RuntimeError("Should not be discarding a session with open connections")
         log.debug("Discarding session %r last in use %r milliseconds ago", session.id, session.milliseconds_since_last_unsubscribe)
@@ -257,7 +252,7 @@ class ApplicationContext(object):
         # session.destroy() wants the document lock so it can shut down the document
         # callbacks.
         def do_discard():
-            # while we yielded for the document lock, the discard-worthiness of the
+            # while we awaited for the document lock, the discard-worthiness of the
             # session may have changed.
             # However, since we have the document lock, our own lock will cause the
             # block count to be 1. If there's any other block count besides our own,
@@ -269,21 +264,19 @@ class ApplicationContext(object):
                 log.trace("Session %r was successfully discarded", session.id)
             else:
                 log.warning("Session %r was scheduled to discard but came back to life", session.id)
-        yield session.with_document_locked(do_discard)
+        await session.with_document_locked(do_discard)
 
         # session lifecycle hooks are supposed to be called outside the document lock,
         # we only run these if we actually ended up destroying the session.
         if session_context.destroyed:
             try:
-                result = self._application.on_session_destroyed(session_context)
-                yield yield_for_all_futures(result)
+                await self._application.on_session_destroyed(session_context)
             except Exception as e:
                 log.error("Failed to run session destroy hooks %r", e, exc_info=True)
 
-        raise gen.Return(None)
+        return None
 
-    @gen.coroutine
-    def _cleanup_sessions(self, unused_session_linger_milliseconds):
+    async def _cleanup_sessions(self, unused_session_linger_milliseconds):
         def should_discard_ignoring_block(session):
             return session.connection_count == 0 and \
                 (session.milliseconds_since_last_unsubscribe > unused_session_linger_milliseconds or \
@@ -299,33 +292,55 @@ class ApplicationContext(object):
         # asynchronously reconsider each session
         for session in to_discard:
             if should_discard_ignoring_block(session) and not session.expiration_blocked:
-                yield self._discard_session(session, should_discard_ignoring_block)
+                await self._discard_session(session, should_discard_ignoring_block)
 
-        raise gen.Return(None)
+        return None
 
 #-----------------------------------------------------------------------------
 # Private API
 #-----------------------------------------------------------------------------
 
-class _RequestProxy(object):
-    def __init__(self, request):
+class _RequestProxy:
+    def __init__(self, request, cookies=None, headers=None):
         self._request = request
 
         arguments = dict(request.arguments)
-        if 'bokeh-protocol-version' in arguments: del arguments['bokeh-protocol-version']
         if 'bokeh-session-id' in arguments: del arguments['bokeh-session-id']
         self._arguments = arguments
+        if cookies is not None:
+            self._cookies = cookies
+        elif hasattr(request, 'cookies'):
+            # Django cookies are plain strings, tornado cookies are objects with a value
+            self._cookies = {k: v if isinstance(v, str) else v.value
+                             for k, v in request.cookies.items()}
+        else:
+            self._cookies = {}
+
+        if headers is not None:
+            self._headers = headers
+        elif hasattr(request, 'headers'):
+            self._headers = dict(request.headers)
+        else:
+            self._headers = {}
 
     @property
     def arguments(self):
         return self._arguments
 
+    @property
+    def cookies(self):
+        return self._cookies
+
+    @property
+    def headers(self):
+        return self._headers
+
     def __getattr__(self, name):
         if not name.startswith("_"):
             val = getattr(self._request, name, None)
             if val is not None:
                 return val
-        return super.__getattr__(name)
+        return super().__getattr__(name)
 
 #-----------------------------------------------------------------------------
 # Code
