@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,23 +12,16 @@ receive.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
 # Imports
 #-----------------------------------------------------------------------------
 
-# Standard library imports
-
-# External imports
-from tornado import gen
-
 # Bokeh imports
-from .session import ServerSession
 from ..protocol.exceptions import ProtocolError
+from .session import ServerSession
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -42,11 +35,10 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-class ProtocolHandler(object):
+class ProtocolHandler:
     ''' A Bokeh server may be expected to receive any of the following protocol
     messages:
 
-    * ``EVENT``
     * ``PATCH-DOC``
     * ``PULL-DOC-REQ``
     * ``PUSH-DOC``
@@ -70,10 +62,8 @@ class ProtocolHandler(object):
         self._handlers['PUSH-DOC'] = ServerSession.push
         self._handlers['PATCH-DOC'] = ServerSession.patch
         self._handlers['SERVER-INFO-REQ'] = self._server_info_req
-        self._handlers['EVENT'] = ServerSession.event
 
-    @gen.coroutine
-    def handle(self, message, connection):
+    async def handle(self, message, connection):
         ''' Delegate a received message to the appropriate handler.
 
         Args:
@@ -88,7 +78,7 @@ class ProtocolHandler(object):
 
         '''
 
-        handler = self._handlers.get((message.msgtype, message.revision))
+        handler = self._handlers.get(message.msgtype)
 
         if handler is None:
             handler = self._handlers.get(message.msgtype)
@@ -97,16 +87,15 @@ class ProtocolHandler(object):
             raise ProtocolError("%s not expected on server" % message)
 
         try:
-            work = yield handler(message, connection)
+            work = await handler(message, connection)
         except Exception as e:
             log.error("error handling message\n message: %r \n error: %r",
                       message, e, exc_info=True)
             work = connection.error(message, repr(e))
-        raise gen.Return(work)
+        return work
 
-    @gen.coroutine
-    def _server_info_req(self, message, connection):
-        raise gen.Return(connection.protocol.create('SERVER-INFO-REPLY', message.header['msgid']))
+    async def _server_info_req(self, message, connection):
+        return connection.protocol.create('SERVER-INFO-REPLY', message.header['msgid'])
 
 #-----------------------------------------------------------------------------
 # Dev API
