@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -620,7 +620,7 @@ class Settings:
 
     # Non-settings methods
 
-    def bokehjsdir(self):
+    def bokehjsdir(self) -> str:
         ''' The location of the BokehJS source tree.
 
         '''
