@@ -1,10 +1,10 @@
-#-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2020, Anaconda, Inc., and Bokeh Contributors.
+# -----------------------------------------------------------------------------
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
-#-----------------------------------------------------------------------------
-''' Simplify linking to Bokeh Github resources.
+# -----------------------------------------------------------------------------
+""" Simplify linking to Bokeh Github resources.
 
 This module proved four new roles that can be uses to easily link to various
 resources in the Bokeh Github repository:
@@ -34,100 +34,103 @@ The repo history shows that :bokeh-commit:`bf19bcb` was made in
 in :bokeh-pull:`1698`,which closed :bokeh-issue:`1694`. This included
 updating all of the files in the :bokeh-tree:`examples` subdirectory.
 
-'''
+"""
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Boilerplate
-#-----------------------------------------------------------------------------
-import logging # isort:skip
+# -----------------------------------------------------------------------------
+import logging  # isort:skip
+
 log = logging.getLogger(__name__)
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Imports
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 # External imports
 from docutils import nodes, utils
 from docutils.parsers.rst.roles import set_classes
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 # Globals and constants
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
 __all__ = (
-    'bokeh_commit',
-    'bokeh_issue',
-    'bokeh_pull',
-    'bokeh_tree',
-    'setup',
+    "bokeh_commit",
+    "bokeh_issue",
+    "bokeh_pull",
+    "bokeh_tree",
+    "setup",
 )
 
-#-----------------------------------------------------------------------------
-# General API
-#-----------------------------------------------------------------------------
+BOKEH_GH = "https://github.com/bokeh/bokeh"
 
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+# General API
+# -----------------------------------------------------------------------------
+
+# -----------------------------------------------------------------------------
 # Dev API
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
+
 
 def bokeh_commit(name, rawtext, text, lineno, inliner, options=None, content=None):
-    ''' Link to a Bokeh Github issue.
+    """Link to a Bokeh Github issue.
 
     Returns 2 part tuple containing list of nodes to insert into the
     document and a list of system messages.  Both are allowed to be
     empty.
 
-    '''
+    """
     app = inliner.document.settings.env.app
-    node = _make_gh_link_node(app, rawtext, 'commit', 'commit ', 'commit', text, options)
+    node = _make_gh_link_node(app, rawtext, "commit", "commit ", "commit", text, options)
     return [node], []
 
+
 def bokeh_issue(name, rawtext, text, lineno, inliner, options=None, content=None):
-    ''' Link to a Bokeh Github issue.
+    """Link to a Bokeh Github issue.
 
     Returns 2 part tuple containing list of nodes to insert into the
     document and a list of system messages.  Both are allowed to be
     empty.
 
-    '''
+    """
     app = inliner.document.settings.env.app
     try:
         issue_num = int(text)
         if issue_num <= 0:
             raise ValueError
     except ValueError:
-        msg = inliner.reporter.error(
-            'Github issue number must be a number greater than or equal to 1; '
-            '"%s" is invalid.' % text, line=lineno)
+        msg = inliner.reporter.error(f"Github issue number must be a number greater than or equal to 1; {text!r} is invalid.", line=lineno)
         prb = inliner.problematic(rawtext, rawtext, msg)
         return [prb], [msg]
-    node = _make_gh_link_node(app, rawtext, 'issue', '#', 'issues', str(issue_num), options)
+    node = _make_gh_link_node(app, rawtext, "issue", "#", "issues", str(issue_num), options)
     return [node], []
 
+
 def bokeh_pull(name, rawtext, text, lineno, inliner, options=None, content=None):
-    ''' Link to a Bokeh Github issue.
+    """Link to a Bokeh Github issue.
 
     Returns 2 part tuple containing list of nodes to insert into the
     document and a list of system messages.  Both are allowed to be
     empty.
 
-    '''
+    """
     app = inliner.document.settings.env.app
     try:
         issue_num = int(text)
         if issue_num <= 0:
             raise ValueError
     except ValueError:
-        msg = inliner.reporter.error(
-            'Github pull request number must be a number greater than or equal to 1; '
-            '"%s" is invalid.' % text, line=lineno)
+        msg = inliner.reporter.error(f"Github pull request number must be a number greater than or equal to 1; {text!r} is invalid.", line=lineno)
         prb = inliner.problematic(rawtext, rawtext, msg)
         return [prb], [msg]
-    node = _make_gh_link_node(app, rawtext, 'pull', 'pull request ', 'pull', str(issue_num), options)
+    node = _make_gh_link_node(app, rawtext, "pull", "pull request ", "pull", str(issue_num), options)
     return [node], []
 
+
 def bokeh_tree(name, rawtext, text, lineno, inliner, options=None, content=None):
-    ''' Link to a URL in the Bokeh GitHub tree, pointing to appropriate tags
+    """Link to a URL in the Bokeh GitHub tree, pointing to appropriate tags
     for releases, or to master otherwise.
 
     The link text is simply the URL path supplied, so typical usage might
@@ -142,34 +145,35 @@ def bokeh_tree(name, rawtext, text, lineno, inliner, options=None, content=None)
     document and a list of system messages.  Both are allowed to be
     empty.
 
-    '''
+    """
     app = inliner.document.settings.env.app
 
-    tag = app.env.config['version']
-    if '-' in tag:
-        tag = 'master'
+    tag = app.env.config["version"]
+    if "-" in tag:
+        tag = "master"
 
-    url = "%s/tree/%s/%s" % (_BOKEH_GH, tag, text)
+    url = f"{BOKEH_GH}/tree/{tag}/{text}"
     options = options or {}
     set_classes(options)
     node = nodes.reference(rawtext, text, refuri=url, **options)
     return [node], []
 
+
 def setup(app):
-    ''' Required Sphinx extension setup function. '''
-    app.add_role('bokeh-commit', bokeh_commit)
-    app.add_role('bokeh-issue', bokeh_issue)
-    app.add_role('bokeh-pull', bokeh_pull)
-    app.add_role('bokeh-tree', bokeh_tree)
+    """ Required Sphinx extension setup function. """
+    app.add_role("bokeh-commit", bokeh_commit)
+    app.add_role("bokeh-issue", bokeh_issue)
+    app.add_role("bokeh-pull", bokeh_pull)
+    app.add_role("bokeh-tree", bokeh_tree)
 
-#-----------------------------------------------------------------------------
+
+# -----------------------------------------------------------------------------
 # Private API
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
 
-_BOKEH_GH = "https://github.com/bokeh/bokeh"
 
 def _make_gh_link_node(app, rawtext, role, kind, api_type, id, options=None):
-    ''' Return a link to a Bokeh Github resource.
+    """Return a link to a Bokeh Github resource.
 
     Args:
         app (Sphinx app) : current app
@@ -180,14 +184,14 @@ def _make_gh_link_node(app, rawtext, role, kind, api_type, id, options=None):
         id : (str) : id of the resource to link to
         options (dict) : options dictionary passed to role function
 
-    '''
-    url = "%s/%s/%s" % (_BOKEH_GH, api_type, id)
+    """
+    url = f"{BOKEH_GH}/{api_type}/{id}"
     options = options or {}
     set_classes(options)
-    node = nodes.reference(
-        rawtext, kind + utils.unescape(id), refuri=url, **options)
+    node = nodes.reference(rawtext, f"{kind}{utils.unescape(id)}", refuri=url, **options)
     return node
 
-#-----------------------------------------------------------------------------
+
+# -----------------------------------------------------------------------------
 # Code
-#-----------------------------------------------------------------------------
+# -----------------------------------------------------------------------------
