@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -12,9 +12,7 @@ interfaces to classes.
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -22,13 +20,11 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-
-# External imports
-from six import string_types
+from inspect import signature
 
 # Bokeh imports
 from ..events import Event
-from ..util.future import get_param_info, format_signature, signature
+from ..util.functions import get_param_info
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -43,21 +39,28 @@ __all__ = (
 # General API
 #-----------------------------------------------------------------------------
 
-class EventCallbackManager(object):
+class EventCallbackManager:
     ''' A mixin class to provide an interface for registering and
     triggering event callbacks on the Python side.
 
     '''
     def __init__(self, *args, **kw):
-        super(EventCallbackManager, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
         self._event_callbacks = dict()
 
     def on_event(self, event, *callbacks):
-        if not isinstance(event, string_types) and issubclass(event, Event):
+        ''' Run callbacks when the specified event occurs on this Model
+
+        Not all Events are supported for all Models.
+        See specific Events in :ref:`bokeh.events` for more information on
+        which Models are able to trigger them.
+        '''
+        if not isinstance(event, str) and issubclass(event, Event):
             event = event.event_name
 
         for callback in callbacks:
-            _check_callback(callback, ('event',), what='Event callback')
+            if _nargs(callback) != 0:
+                _check_callback(callback, ('event',), what='Event callback')
 
         if event not in self._event_callbacks:
             self._event_callbacks[event] = [cb for cb in callbacks]
@@ -68,9 +71,26 @@ class EventCallbackManager(object):
             self.subscribed_events.append(event)
 
     def _trigger_event(self, event):
-        for callback in self._event_callbacks.get(event.event_name,[]):
-            if event._model_id is not None and self.id == event._model_id:
-                callback(event)
+        def invoke():
+            for callback in self._event_callbacks.get(event.event_name,[]):
+                if event._model_id is not None and self.id == event._model_id:
+                    if _nargs(callback) == 0:
+                        callback()
+                    else:
+                        callback(event)
+
+        # TODO: here we might mirror the property callbacks and have something
+        # like Document._notify_event which creates an *internal* Bokeh event
+        # (for the user event, confusing!) that then dispatches in the document
+        # and applies curdoc wrapper there. However, most of that machinery is
+        # to support the bi-directionality of property changes. Currently (user)
+        # events only run from client to server. Would like to see if some of the
+        # internal eventing can be reduced or simplified in general before
+        # plugging more into it. For now, just handle the curdoc bits here.
+        if hasattr(self, '_document') and self._document is not None:
+            self._document._with_self_as_curdoc(invoke)
+        else:
+            invoke()
 
     def _update_event_callbacks(self):
         if self.document is None:
@@ -80,14 +100,14 @@ class EventCallbackManager(object):
             self.document._subscribed_models[key].add(self)
 
 
-class PropertyCallbackManager(object):
+class PropertyCallbackManager:
     ''' A mixin class to provide an interface for registering and
     triggering callbacks.
 
     '''
 
     def __init__(self, *args, **kw):
-        super(PropertyCallbackManager, self).__init__(*args, **kw)
+        super().__init__(*args, **kw)
         self._callbacks = dict()
 
     def on_change(self, attr, *callbacks):
@@ -152,15 +172,21 @@ class PropertyCallbackManager(object):
 # Private API
 #-----------------------------------------------------------------------------
 
+def _nargs(fn):
+    sig = signature(fn)
+    all_names, default_values = get_param_info(sig)
+    return len(all_names) - len(default_values)
+
 def _check_callback(callback, fargs, what="Callback functions"):
     '''Bokeh-internal function to check callback signature'''
     sig = signature(callback)
-    formatted_args = format_signature(sig)
+    formatted_args = str(sig)
     error_msg = what + " must have signature func(%s), got func%s"
 
     all_names, default_values = get_param_info(sig)
 
-    if len(all_names) - len(default_values) != len(fargs):
+    nargs = len(all_names) - len(default_values)
+    if nargs != len(fargs):
         raise ValueError(error_msg % (", ".join(fargs), formatted_args))
 
 #-----------------------------------------------------------------------------
