@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2019, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) 2012 - 2021, Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -11,9 +11,7 @@
 #-----------------------------------------------------------------------------
 # Boilerplate
 #-----------------------------------------------------------------------------
-from __future__ import absolute_import, division, print_function, unicode_literals
-
-import logging
+import logging # isort:skip
 log = logging.getLogger(__name__)
 
 #-----------------------------------------------------------------------------
@@ -21,18 +19,14 @@ log = logging.getLogger(__name__)
 #-----------------------------------------------------------------------------
 
 # Standard library imports
-import io
+import hashlib
 import json
 import os
-from os.path import dirname, join, abspath, exists, isabs
 import re
-from subprocess import Popen, PIPE
-from collections import OrderedDict
 import sys
-
-# External imports
-import hashlib
-import six
+from collections import OrderedDict
+from os.path import abspath, dirname, exists, isabs, join
+from subprocess import PIPE, Popen
 
 # Bokeh imports
 from ..model import Model
@@ -48,7 +42,6 @@ __all__ = (
     'bundle_all_models',
     'bundle_models',
     'calc_cache_key',
-    'CoffeeScript',
     'CompilationError',
     'CustomModel',
     'FromFile',
@@ -80,7 +73,7 @@ class CompilationError(RuntimeError):
 
     '''
     def __init__(self, error):
-        super(CompilationError, self).__init__()
+        super().__init__()
         if isinstance(error, dict):
             self.line = error.get("line")
             self.column = error.get("column")
@@ -94,7 +87,7 @@ class CompilationError(RuntimeError):
         return "\n" + self.text.strip()
 
 bokehjs_dir = settings.bokehjsdir()
-nodejs_min_version = (6, 10, 0)
+nodejs_min_version = (14, 0, 0)
 
 def nodejs_version():
     return _version(_run_nodejs)
@@ -103,10 +96,6 @@ def npmjs_version():
     return _version(_run_npmjs)
 
 def nodejs_compile(code, lang="javascript", file=None):
-    if lang == "coffeescript":
-        from bokeh.util.deprecation import deprecated
-        deprecated("CoffeeScript support is deprecated and will be removed in an eventual 2.0 release. "
-                   "Use JavaScript or TypeScript directly instead.")
     compilejs_script = join(bokehjs_dir, "js", "compiler.js")
     output = _run_nodejs([compilejs_script], dict(code=code, lang=lang, file=file, bokehjs_dir=bokehjs_dir))
     lines = output.split("\n")
@@ -121,7 +110,7 @@ def nodejs_compile(code, lang="javascript", file=None):
     else:
         return dict(error=obj)
 
-class Implementation(object):
+class Implementation:
     ''' Base class for representing Bokeh custom model implementations.
 
     '''
@@ -143,36 +132,6 @@ class Inline(Implementation):
         self.code = code
         self.file = file
 
-class CoffeeScript(Inline):
-    ''' An implementation for a Bokeh custom model in CoffeeScript.
-
-    Example:
-
-        .. code-block:: python
-
-            class MyExt(Model):
-                __implementation__ = CoffeeScript(""" <CoffeeScript code> """)
-
-    Note that ``CoffeeScript`` is the default implementation language for
-    custom model implementations. The following is equivalent to example above:
-
-    .. code-block:: python
-
-        class MyExt(Model):
-            __implementation__ == """ <some coffeescript code> """
-
-    '''
-
-    def __init__(self, *args, **kw):
-        from bokeh.util.deprecation import deprecated
-        deprecated("CoffeeScript support is deprecated and will be removed in an eventual 2.0 release. "
-                   "Use JavaScript or TypeScript directly instead.")
-        super(CoffeeScript, self).__init__(*args, **kw)
-
-    @property
-    def lang(self):
-        return "coffeescript"
-
 class TypeScript(Inline):
     ''' An implementation for a Bokeh custom model in TypeScript
 
@@ -220,14 +179,12 @@ class FromFile(Implementation):
 
     '''
     def __init__(self, path):
-        with io.open(path, encoding="utf-8") as f:
+        with open(path, encoding="utf-8") as f:
             self.code = f.read()
         self.file = path
 
     @property
     def lang(self):
-        if self.file.endswith(".coffee"):
-            return "coffeescript"
         if self.file.endswith(".ts"):
             return "typescript"
         if self.file.endswith(".js"):
@@ -236,9 +193,9 @@ class FromFile(Implementation):
             return "less"
 
 #: recognized extensions that can be compiled
-exts = (".coffee", ".ts", ".js", ".css", ".less")
+exts = (".ts", ".js", ".css", ".less")
 
-class CustomModel(object):
+class CustomModel:
     ''' Represent a custom (user-defined) Bokeh model.
 
     '''
@@ -278,11 +235,11 @@ class CustomModel(object):
     def implementation(self):
         impl = self.cls.__implementation__
 
-        if isinstance(impl, six.string_types):
+        if isinstance(impl, str):
             if "\n" not in impl and impl.endswith(exts):
                 impl = FromFile(impl if isabs(impl) else join(self.path, impl))
             else:
-                impl = CoffeeScript(impl)
+                impl = TypeScript(impl)
 
         if isinstance(impl, Inline) and impl.file is None:
             file = "%s%s.ts" % (self.file + ":" if self.file else "", self.name)
@@ -445,7 +402,7 @@ def _detect_nodejs():
 _nodejs = None
 _npmjs = None
 
-def _nodejs_path():
+def _nodejs_path() -> str:
     global _nodejs
     if _nodejs is None:
         _nodejs = _detect_nodejs()
@@ -539,7 +496,7 @@ def _bundle_models(custom_models):
     exports = []
     modules = []
 
-    with io.open(join(bokehjs_dir, "js", "bokeh.json"), encoding="utf-8") as f:
+    with open(join(bokehjs_dir, "js", "bokeh.json"), encoding="utf-8") as f:
         bokeh = json.loads(f.read())
 
     known_modules = set()
@@ -600,7 +557,7 @@ def _bundle_models(custom_models):
         return resolved
 
     def resolve_deps(deps, root):
-        custom_modules = set(model.module for model in custom_models.values())
+        custom_modules = {model.module for model in custom_models.values()}
         missing = set(deps) - known_modules - custom_modules
         return resolve_modules(missing, root)
 
